#include <iostream>
#include <fstream>
#include <string>
#include <sstream>
#include <stdlib.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_eigen.h>

using namespace std;


//
// Returns true if the input matrix is Hermitian.
//
bool IsHermitian(const gsl_matrix_complex * const M)
{
	gsl_matrix_complex *N;
	gsl_complex zm, zn;
	int i, j;
	bool isHerm = true;

	N = gsl_matrix_complex_alloc(M->size1, M->size2);

	for(i = 0; i < M->size1; ++i)
	{
		for(j = 0; j < M->size2; ++j)
		{
			zm = gsl_matrix_complex_get(M, i, j);
			GSL_SET_COMPLEX(&zn, GSL_REAL(zm), ((-1)*GSL_IMAG(zm)));
			gsl_matrix_complex_set(N, i, j, zn);
		}
	}

	gsl_matrix_complex_transpose(N);

	for(i = 0; i < M->size1; ++i)
	{
		for(j = 0; j < M->size2; ++j)
		{
			zm = gsl_matrix_complex_get(M, i, j);
			zn = gsl_matrix_complex_get(N, i, j);

			if( GSL_REAL(zm) != GSL_REAL(zn) ||
			    GSL_IMAG(zm) != GSL_IMAG(zn) )
			{
				isHerm = false;
				goto _exit;
			}
		}
	}

_exit:
	gsl_matrix_complex_free(N);

	return isHerm;
}

int main()
{	
	ifstream infile("buginput.txt");

	if(infile.is_open() == false)
	{
		cout << "Error opening file." << endl;
		return 0;
	}

	string indata;
	string token;
	int x, y, i, j;
	float real, imag;

	// Read matrix size
	getline(infile, indata);
	istringstream iss(indata);
	getline(iss, token, ','); x = atoi(token.c_str());
	getline(iss, token, ','); y = atoi(token.c_str());
	if( x != y )
	{
		cout << "Error. Not square matrix." << endl;
		return 0;
	}

	// Read matrix data
	gsl_matrix_complex *M = gsl_matrix_complex_alloc(x, y);
	gsl_complex z;
	for(i = 0; i < x; ++i)
	{
		for(j = 0; j < y; ++j)
		{
			getline(infile, indata);
			istringstream iss(indata);

			getline(iss, token, ','); real = atof(token.c_str());
			getline(iss, token, ','); imag = atof(token.c_str());

			//cout << real << "+" << imag << "\t";

			GSL_SET_COMPLEX(&z, real, imag);
			gsl_matrix_complex_set(M, i, j, z);
		}
	}
	infile.close();

	if( IsHermitian(M) == false )
	{
		cout << "Error. Matrix is not Hermitian. Cannot apply gsl_eigen_hermv." << endl;
		return 0;
	}
	cout << "Read Hermitian matrix." << endl;

	// Call gsl_eigen_hermv
	gsl_vector *eval		= gsl_vector_alloc(M->size1);
	gsl_matrix_complex *evec	= gsl_matrix_complex_alloc(M->size1, M->size2);
	gsl_eigen_hermv_workspace *w	= gsl_eigen_hermv_alloc(M->size1);

	cout << "gsl_eigen_hermv returned " << gsl_eigen_hermv(M, eval, evec, w) << "." << endl;

	// Write evec to file
	ofstream outfile("bugoutput.txt");
	for(i = 0; i < x; ++i)
	{
		for(j = 0; j < y; ++j)
		{
			z = gsl_matrix_complex_get(evec, i, j);

			outfile << GSL_REAL(z) << " + " << GSL_IMAG(z) << "\t";
		}
		outfile << endl;
	}
	outfile.close();

	// Free resources
	gsl_eigen_hermv_free(w);
	gsl_matrix_complex_free(evec);
	gsl_vector_free(eval);
	gsl_matrix_complex_free(M);


	return 0;
}
